@@ -69,11 +69,14 @@ handle_welcome_req(Req, _) ->
 
 get_features() ->
     case dreyfus:available() of
-        true ->
-            [search | config:features()];
-        false ->
-            config:features()
-    end.
+        true -> [search];
+        false -> []
+    end ++
+        case nouveau:enabled() of
+            true -> [nouveau];
+            false -> []
+        end ++
+        config:features().
 
 handle_favicon_req(Req) ->
     handle_favicon_req(Req, get_docroot()).
@@ -216,14 +219,8 @@ handle_dbs_info_req(Req) ->
 
 handle_task_status_req(#httpd{method = 'GET'} = Req) ->
     ok = chttpd:verify_is_server_admin(Req),
-    {Replies, _BadNodes} = gen_server:multi_call(couch_task_status, all),
-    Response = lists:flatmap(
-        fun({Node, Tasks}) ->
-            [{[{node, Node} | Task]} || Task <- Tasks]
-        end,
-        Replies
-    ),
-    send_json(Req, lists:sort(Response));
+    TaskList = chttpd_util:get_active_tasks(nodes([visible, this])),
+    send_json(Req, lists:sort(TaskList));
 handle_task_status_req(Req) ->
     send_method_not_allowed(Req, "GET,HEAD").
 
@@ -292,7 +289,7 @@ choose_node(Key) when is_binary(Key) ->
     Nodes = lists:sort([node() | erlang:nodes()]),
     lists:nth(1 + Checksum rem length(Nodes), Nodes);
 choose_node(Key) ->
-    choose_node(term_to_binary(Key)).
+    choose_node(?term_to_bin(Key)).
 
 handle_reload_query_servers_req(#httpd{method = 'POST'} = Req) ->
     chttpd:validate_ctype(Req, "application/json"),
@@ -307,16 +304,14 @@ handle_uuids_req(Req) ->
 handle_up_req(#httpd{method = 'GET'} = Req) ->
     case config:get("couchdb", "maintenance_mode") of
         "true" ->
-            send_json(Req, 404, {[{status, maintenance_mode}]});
+            send_json(Req, 404, #{status => maintenance_mode});
         "nolb" ->
-            send_json(Req, 404, {[{status, nolb}]});
+            send_json(Req, 404, #{status => nolb});
         _ ->
-            {ok, {Status}} = mem3_seeds:get_status(),
-            case couch_util:get_value(status, Status) of
-                ok ->
-                    send_json(Req, 200, {Status});
-                seeding ->
-                    send_json(Req, 404, {Status})
+            {ok, #{} = Status} = mem3_seeds:get_status(),
+            case Status of
+                #{status := ok} -> send_json(Req, 200, Status);
+                #{status := seeding} -> send_json(Req, 404, Status)
             end
     end;
 handle_up_req(Req) ->
