@@ -43,7 +43,15 @@
     restart/1,
     restart_busy/2,
     restart_busy/3,
-    restart_busy/4
+    restart_busy/4,
+    dead_nodes/0,
+    dead_nodes/1,
+    ping/1,
+    ping/2,
+    ping_nodes/0,
+    ping_nodes/1,
+    ping_nodes/2,
+    node_events/0
 ]).
 
 -export([
@@ -85,7 +93,10 @@ help() ->
         print_report,
         print_report_with_info_width,
         restart,
-        restart_busy
+        restart_busy,
+        dead_nodes,
+        ping_nodes,
+        node_events
     ].
 
 -spec help(Function :: function_name()) -> ok.
@@ -435,6 +446,49 @@ help(analyze_resource_hoggers) ->
         along with the number of top processes to include in result, TopN. See `couch_debug:help(resource_hoggers_snapshot)` for an 
         example and more info.
 
+        ---
+    ", []);
+help(dead_nodes) ->
+    io:format("
+        dead_nodes()
+        dead_nodes(Timeout)
+        --------------------------------
+
+        Get the list of 'dead' nodes, that is node which appears in the
+        mem3:nodes() list but are not connected to some nodes of the cluster.
+
+        ---
+    ", []);
+help(ping) ->
+    io:format("
+        ping(Node)
+        ping(Node, Timeout)
+        --------------------------------
+
+        Ping a node and return either a time in microseconds or an error term.
+
+        ---
+    ", []);
+help(ping_nodes) ->
+    io:format("
+        ping_nodes()
+        ping_nodes(Timeout)
+        ping_nodes(Nodes, Timeout)
+        --------------------------------
+
+        Ping the list of currently connected nodes. Return a list of {Node,
+        Result} tuples where Result is either a time in microseconds or an
+        error term.
+
+        ---
+    ", []);
+help(node_events) ->
+    io:format("
+        node_events()
+        --------------------------------
+
+        Return the list of nodeup/nodedown events for each node in the cluster.
+
         ---
     ", []);
 help(Unknown) ->
@@ -910,6 +964,30 @@ restart_busy(ProcessList, Threshold, DelayInMsec, Property) when
         busy(ProcessList, Threshold, Property)
     ).
 
+dead_nodes() ->
+    mem3:dead_nodes().
+
+dead_nodes(Timeout) ->
+    mem3:dead_nodes(Timeout).
+
+ping(Node) ->
+    mem3:ping(Node).
+
+ping(Node, Timeout) ->
+    mem3:ping(Node, Timeout).
+
+ping_nodes() ->
+    mem3:ping_nodes().
+
+ping_nodes(Timeout) ->
+    mem3:ping_nodes(Timeout).
+
+ping_nodes(Nodes, Timeout) ->
+    mem3:ping_nodes(Nodes, Timeout).
+
+node_events() ->
+    mem3_distribution:events().
+
 %% Pretty print functions
 
 %% Limitations:
@@ -1046,7 +1124,7 @@ random_processes(Acc, Depth) ->
                 end);
             open_port ->
                 spawn_link(fun() ->
-                    Port = erlang:open_port({spawn, "sleep 10"}, []),
+                    Port = erlang:open_port({spawn, "sleep 10"}, [hide]),
                     true = erlang:link(Port),
                     Caller ! {Ref, random_processes(Depth - 1)},
                     receive
@@ -1059,7 +1137,7 @@ random_processes(Acc, Depth) ->
     end.
 
 oneof(Options) ->
-    lists:nth(couch_rand:uniform(length(Options)), Options).
+    lists:nth(rand:uniform(length(Options)), Options).
 
 tree() ->
     [InitialPid | _] = Processes = random_processes(5),
@@ -1069,7 +1147,13 @@ setup() ->
     tree().
 
 teardown({_InitialPid, Processes, _Tree}) ->
-    [exit(Pid, normal) || Pid <- Processes].
+    [
+        begin
+            (catch unlink(Pid)),
+            exit(Pid, kill)
+        end
+     || Pid <- Processes
+    ].
 
 link_tree_test_() ->
     {
