@@ -70,67 +70,86 @@ read_write_test_() ->
         "Common file read/write tests",
         {
             setup,
-            fun() -> test_util:start(?MODULE, [ioq]) end,
-            fun test_util:stop/1,
-            ?foreach([
-                fun should_increase_file_size_on_write/1,
-                fun should_return_current_file_size_on_write/1,
-                fun should_write_and_read_term/1,
-                fun should_write_and_read_binary/1,
-                fun should_write_and_read_large_binary/1,
-                fun should_return_term_as_binary_for_reading_binary/1,
-                fun should_read_term_written_as_binary/1,
-                fun should_read_iolist/1,
-                fun should_fsync/1,
-                fun should_not_read_beyond_eof/1,
-                fun should_truncate/1
-            ])
+            fun() -> test_util:start_couch() end,
+            fun test_util:stop_couch/1,
+            {
+                foreach,
+                fun setup/0,
+                fun teardown/1,
+                [
+                    ?TDEF_FE(should_increase_file_size_on_write),
+                    ?TDEF_FE(should_return_current_file_size_on_write),
+                    ?TDEF_FE(should_write_and_read_term),
+                    ?TDEF_FE(should_write_and_read_binary),
+                    ?TDEF_FE(should_write_and_read_large_binary),
+                    ?TDEF_FE(should_return_term_as_binary_for_reading_binary),
+                    ?TDEF_FE(should_read_term_written_as_binary),
+                    ?TDEF_FE(should_read_iolist),
+                    ?TDEF_FE(should_fsync),
+                    ?TDEF_FE(should_update_fsync_stats),
+                    ?TDEF_FE(should_not_read_beyond_eof),
+                    ?TDEF_FE(should_truncate)
+                ]
+            }
         }
     }.
 
 should_increase_file_size_on_write(Fd) ->
     {ok, 0, _} = couch_file:append_term(Fd, foo),
     {ok, Size} = couch_file:bytes(Fd),
-    ?_assert(Size > 0).
+    ?assert(Size > 0).
 
 should_return_current_file_size_on_write(Fd) ->
     {ok, 0, _} = couch_file:append_term(Fd, foo),
     {ok, Size} = couch_file:bytes(Fd),
-    ?_assertMatch({ok, Size, _}, couch_file:append_term(Fd, bar)).
+    ?assertMatch({ok, Size, _}, couch_file:append_term(Fd, bar)).
 
 should_write_and_read_term(Fd) ->
     {ok, Pos, _} = couch_file:append_term(Fd, foo),
-    ?_assertMatch({ok, foo}, couch_file:pread_term(Fd, Pos)).
+    ?assertMatch({ok, foo}, couch_file:pread_term(Fd, Pos)).
 
 should_write_and_read_binary(Fd) ->
     {ok, Pos, _} = couch_file:append_binary(Fd, <<"fancy!">>),
-    ?_assertMatch({ok, <<"fancy!">>}, couch_file:pread_binary(Fd, Pos)).
+    ?assertMatch({ok, <<"fancy!">>}, couch_file:pread_binary(Fd, Pos)).
 
 should_return_term_as_binary_for_reading_binary(Fd) ->
     {ok, Pos, _} = couch_file:append_term(Fd, foo),
     Foo = couch_compress:compress(foo, snappy),
-    ?_assertMatch({ok, Foo}, couch_file:pread_binary(Fd, Pos)).
+    ?assertMatch({ok, Foo}, couch_file:pread_binary(Fd, Pos)).
 
 should_read_term_written_as_binary(Fd) ->
     {ok, Pos, _} = couch_file:append_binary(Fd, <<131, 100, 0, 3, 102, 111, 111>>),
-    ?_assertMatch({ok, foo}, couch_file:pread_term(Fd, Pos)).
+    ?assertMatch({ok, foo}, couch_file:pread_term(Fd, Pos)).
 
 should_write_and_read_large_binary(Fd) ->
     BigBin = list_to_binary(lists:duplicate(100000, 0)),
     {ok, Pos, _} = couch_file:append_binary(Fd, BigBin),
-    ?_assertMatch({ok, BigBin}, couch_file:pread_binary(Fd, Pos)).
+    ?assertMatch({ok, BigBin}, couch_file:pread_binary(Fd, Pos)).
 
 should_read_iolist(Fd) ->
     %% append_binary == append_iolist?
     %% Possible bug in pread_iolist or iolist() -> append_binary
     {ok, Pos, _} = couch_file:append_binary(Fd, ["foo", $m, <<"bam">>]),
     {ok, IoList} = couch_file:pread_iolist(Fd, Pos),
-    ?_assertMatch(<<"foombam">>, iolist_to_binary(IoList)).
+    ?assertMatch(<<"foombam">>, iolist_to_binary(IoList)).
 
 should_fsync(Fd) ->
-    {"How does on test fsync?", ?_assertMatch(ok, couch_file:sync(Fd))}.
+    ?assertMatch(ok, couch_file:sync(Fd)).
 
-should_not_read_beyond_eof(Fd) ->
+should_update_fsync_stats(Fd) ->
+    Count0 = couch_stats:sample([fsync, count]),
+    Seq = lists:seq(1, 10),
+    lists:foreach(fun(_) -> ok = couch_file:sync(Fd) end, Seq),
+    Hist = couch_stats:sample([fsync, time]),
+    Count1 = couch_stats:sample([fsync, count]),
+    ?assert(Count1 > Count0),
+    HistMax = proplists:get_value(max, Hist),
+    HistPct = proplists:get_value(percentile, Hist),
+    ?assert(HistMax > 0),
+    ?assertMatch([{50, P50} | _] when P50 > 0, HistPct).
+
+should_not_read_beyond_eof(_) ->
+    {ok, Fd} = couch_file:open(?tempfile(), [create, overwrite]),
     BigBin = list_to_binary(lists:duplicate(100000, 0)),
     DoubleBin = round(byte_size(BigBin) * 2),
     {ok, Pos, _Size} = couch_file:append_binary(Fd, BigBin),
@@ -140,8 +159,10 @@ should_not_read_beyond_eof(Fd) ->
     ok = file:pwrite(Io, Pos, <<0:1/integer, DoubleBin:31/integer>>),
     file:close(Io),
     unlink(Fd),
-    ExpectedError = {badmatch, {'EXIT', {bad_return_value, {read_beyond_eof, Filepath}}}},
-    ?_assertError(ExpectedError, couch_file:pread_binary(Fd, Pos)).
+    ExpectExit = {bad_return_value, {read_beyond_eof, Filepath}},
+    ExpectError = {badmatch, {'EXIT', ExpectExit}},
+    ?assertError(ExpectError, couch_file:pread_binary(Fd, Pos)),
+    catch file:close(Fd).
 
 should_truncate(Fd) ->
     {ok, 0, _} = couch_file:append_term(Fd, foo),
@@ -149,7 +170,7 @@ should_truncate(Fd) ->
     BigBin = list_to_binary(lists:duplicate(100000, 0)),
     {ok, _, _} = couch_file:append_binary(Fd, BigBin),
     ok = couch_file:truncate(Fd, Size),
-    ?_assertMatch({ok, foo}, couch_file:pread_term(Fd, 0)).
+    ?assertMatch({ok, foo}, couch_file:pread_term(Fd, 0)).
 
 pread_limit_test_() ->
     {
@@ -157,31 +178,39 @@ pread_limit_test_() ->
         {
             setup,
             fun() ->
-                Ctx = test_util:start(?MODULE),
-                config:set("couchdb", "max_pread_size", "50000"),
+                Ctx = test_util:start_couch([ioq]),
+                config:set("couchdb", "max_pread_size", "50000", _Persist = false),
                 Ctx
             end,
             fun(Ctx) ->
-                config:delete("couchdb", "max_pread_size"),
-                test_util:stop(Ctx)
+                config:delete("couchdb", "max_pread_size", _Persist = false),
+                test_util:stop_couch(Ctx)
             end,
-            ?foreach([
-                fun should_increase_file_size_on_write/1,
-                fun should_return_current_file_size_on_write/1,
-                fun should_write_and_read_term/1,
-                fun should_write_and_read_binary/1,
-                fun should_not_read_more_than_pread_limit/1
-            ])
+            {
+                foreach,
+                fun setup/0,
+                fun teardown/1,
+                [
+                    ?TDEF_FE(should_increase_file_size_on_write),
+                    ?TDEF_FE(should_return_current_file_size_on_write),
+                    ?TDEF_FE(should_write_and_read_term),
+                    ?TDEF_FE(should_write_and_read_binary),
+                    ?TDEF_FE(should_not_read_more_than_pread_limit)
+                ]
+            }
         }
     }.
 
-should_not_read_more_than_pread_limit(Fd) ->
+should_not_read_more_than_pread_limit(_) ->
+    {ok, Fd} = couch_file:open(?tempfile(), [create, overwrite]),
     {_, Filepath} = couch_file:process_info(Fd),
     BigBin = list_to_binary(lists:duplicate(100000, 0)),
     {ok, Pos, _Size} = couch_file:append_binary(Fd, BigBin),
     unlink(Fd),
-    ExpectedError = {badmatch, {'EXIT', {bad_return_value, {exceed_pread_limit, Filepath, 50000}}}},
-    ?_assertError(ExpectedError, couch_file:pread_binary(Fd, Pos)).
+    ExpectExit = {bad_return_value, {exceed_pread_limit, Filepath, 50000}},
+    ExpectError = {badmatch, {'EXIT', ExpectExit}},
+    ?assertError(ExpectError, couch_file:pread_binary(Fd, Pos)),
+    catch file:close(Fd).
 
 header_test_() ->
     {
@@ -311,14 +340,14 @@ check_header_recovery(CheckFun) ->
     ok.
 
 write_random_data(Fd) ->
-    write_random_data(Fd, 100 + couch_rand:uniform(1000)).
+    write_random_data(Fd, 100 + rand:uniform(1000)).
 
 write_random_data(Fd, 0) ->
     {ok, Bytes} = couch_file:bytes(Fd),
     {ok, (1 + Bytes div ?BLOCK_SIZE) * ?BLOCK_SIZE};
 write_random_data(Fd, N) ->
     Choices = [foo, bar, <<"bizzingle">>, "bank", ["rough", stuff]],
-    Term = lists:nth(couch_rand:uniform(4) + 1, Choices),
+    Term = lists:nth(rand:uniform(4) + 1, Choices),
     {ok, _, _} = couch_file:append_term(Fd, Term),
     write_random_data(Fd, N - 1).
 
@@ -382,7 +411,7 @@ make_enable_recovery_test_case({RootDir, File}, EnableRecovery, Context) ->
         ("couchdb", "delete_after_rename", _) -> false
     end),
     FileExistsBefore = filelib:is_regular(File),
-    couch_file:delete(RootDir, File, [{context, Context}]),
+    DeleteResult = couch_file:delete(RootDir, File, [{context, Context}]),
     FileExistsAfter = filelib:is_regular(File),
     RenamedFiles = filelib:wildcard(filename:rootname(File) ++ "*.deleted.*"),
     DeletedFiles = filelib:wildcard(RootDir ++ "/.delete/*"),
@@ -392,6 +421,7 @@ make_enable_recovery_test_case({RootDir, File}, EnableRecovery, Context) ->
             true -> {0, 1}
         end,
     [
+        ?_assertEqual(ok, DeleteResult),
         ?_assert(FileExistsBefore),
         ?_assertNot(FileExistsAfter),
         ?_assertEqual(ExpectRenamedCount, length(RenamedFiles)),
@@ -436,7 +466,7 @@ nuke_dir_test_() ->
                     File0 = ?tempfile() ++ ".couch",
                     RootDir = filename:dirname(File0),
                     BaseName = filename:basename(File0),
-                    Seed = couch_rand:uniform(8999999999) + 999999999,
+                    Seed = rand:uniform(8999999999) + 999999999,
                     DDocDir = io_lib:format("db.~b_design", [Seed]),
                     ViewDir = filename:join([RootDir, DDocDir]),
                     file:make_dir(ViewDir),
@@ -551,3 +581,182 @@ fake_fsync_fd() ->
         {'$gen_call', From, sync} ->
             gen:reply(From, {error, eio})
     end.
+
+checksum_test_() ->
+    {
+        foreach,
+        fun setup_checksum/0,
+        fun teardown_checksum/1,
+        [
+            ?TDEF_FE(t_write_read_xxhash_checksums),
+            ?TDEF_FE(t_downgrade_xxhash_checksums),
+            ?TDEF_FE(t_read_legacy_checksums_after_upgrade),
+            ?TDEF_FE(t_can_detect_block_corruption_with_xxhash),
+            ?TDEF_FE(t_can_detect_block_corruption_with_legacy_checksum)
+        ]
+    }.
+
+setup_checksum() ->
+    Path = ?tempfile(),
+    Ctx = test_util:start_couch(),
+    config:set("couchdb", "write_xxhash_checksums", "false", _Persist = false),
+    {Ctx, Path}.
+
+teardown_checksum({Ctx, Path}) ->
+    file:delete(Path),
+    meck:unload(),
+    test_util:stop_couch(Ctx),
+    couch_file:reset_checksum_persistent_term_config().
+
+t_write_read_xxhash_checksums({_Ctx, Path}) ->
+    enable_xxhash(),
+
+    {ok, Fd} = couch_file:open(Path, [create]),
+    Header = header,
+    ok = couch_file:write_header(Fd, Header),
+    Bin = <<"bin">>,
+    Chunk = couch_file:assemble_file_chunk_and_checksum(Bin),
+    {ok, Pos, _} = couch_file:append_raw_chunk(Fd, Chunk),
+    couch_file:close(Fd),
+
+    {ok, Fd1} = couch_file:open(Path, []),
+    {ok, Header1} = couch_file:read_header(Fd1),
+    ?assertEqual(Header, Header1),
+    {ok, Bin1} = couch_file:pread_binary(Fd1, Pos),
+    ?assertEqual(Bin, Bin1),
+    ?assertEqual(0, legacy_stats()),
+    couch_file:close(Fd1).
+
+t_downgrade_xxhash_checksums({_Ctx, Path}) ->
+    % We're in the future and writting xxhash checkums by default
+    enable_xxhash(),
+    {ok, Fd} = couch_file:open(Path, [create]),
+    Header = header,
+    ok = couch_file:write_header(Fd, Header),
+    Bin = <<"bin">>,
+    Chunk = couch_file:assemble_file_chunk_and_checksum(Bin),
+    {ok, Pos, _} = couch_file:append_raw_chunk(Fd, Chunk),
+    couch_file:close(Fd),
+
+    % The future was broken, we travel back, but still know how to
+    % interpret future checksums without crashing
+    disable_xxhash(),
+    {ok, Fd1} = couch_file:open(Path, []),
+    {ok, Header1} = couch_file:read_header(Fd1),
+    ?assertEqual(Header, Header1),
+    {ok, Bin1} = couch_file:pread_binary(Fd1, Pos),
+    ?assertEqual(Bin, Bin1),
+
+    % We'll write some legacy checksums to the file and then ensure
+    % we can read both legacy and the new ones
+    OtherBin = <<"otherbin">>,
+    OtherChunk = couch_file:assemble_file_chunk_and_checksum(OtherBin),
+    {ok, OtherPos, _} = couch_file:append_raw_chunk(Fd1, OtherChunk),
+    couch_file:close(Fd1),
+
+    {ok, Fd2} = couch_file:open(Path, []),
+    {ok, Header2} = couch_file:read_header(Fd2),
+    ?assertEqual(Header, Header2),
+    {ok, Bin2} = couch_file:pread_binary(Fd2, Pos),
+    {ok, OtherBin1} = couch_file:pread_binary(Fd2, OtherPos),
+    ?assertEqual(Bin, Bin2),
+    ?assertEqual(OtherBin, OtherBin1),
+    couch_file:close(Fd2).
+
+t_read_legacy_checksums_after_upgrade({_Ctx, Path}) ->
+    % We're in the past and writting legacy checkums by default
+    disable_xxhash(),
+    {ok, Fd} = couch_file:open(Path, [create]),
+    Header = header,
+    ok = couch_file:write_header(Fd, Header),
+    Bin = <<"bin">>,
+    Chunk = couch_file:assemble_file_chunk_and_checksum(Bin),
+    {ok, Pos, _} = couch_file:append_raw_chunk(Fd, Chunk),
+    couch_file:close(Fd),
+
+    % We upgrade and xxhash checksums are now the default, but we can
+    % still read legacy checksums.
+    enable_xxhash(),
+    {ok, Fd1} = couch_file:open(Path, []),
+    {ok, Header1} = couch_file:read_header(Fd1),
+    ?assertEqual(Header, Header1),
+    {ok, Bin1} = couch_file:pread_binary(Fd1, Pos),
+    ?assertEqual(Bin, Bin1),
+    % one header, one chunk
+    ?assertEqual(2, legacy_stats()),
+
+    % We'll write some new checksums to the file and then ensure
+    % we can read both legacy and the new ones
+    OtherBin = <<"otherbin">>,
+    OtherChunk = couch_file:assemble_file_chunk_and_checksum(OtherBin),
+    {ok, OtherPos, _} = couch_file:append_raw_chunk(Fd1, OtherChunk),
+    couch_file:close(Fd1),
+
+    couch_stats:decrement_counter([couchdb, legacy_checksums], legacy_stats()),
+    {ok, Fd2} = couch_file:open(Path, []),
+    {ok, Header2} = couch_file:read_header(Fd2),
+    ?assertEqual(Header, Header2),
+    {ok, Bin2} = couch_file:pread_binary(Fd2, Pos),
+    {ok, OtherBin1} = couch_file:pread_binary(Fd2, OtherPos),
+    ?assertEqual(Bin, Bin2),
+    ?assertEqual(OtherBin, OtherBin1),
+    % one header, legacy chunk, not counting new chunk
+    ?assertEqual(2, legacy_stats()),
+    couch_file:close(Fd2).
+
+t_can_detect_block_corruption_with_xxhash({_Ctx, Path}) ->
+    enable_xxhash(),
+
+    {ok, Fd} = couch_file:open(Path, [create]),
+    Bin = crypto:strong_rand_bytes(100000),
+    Chunk = couch_file:assemble_file_chunk_and_checksum(Bin),
+    {ok, Pos, _} = couch_file:append_raw_chunk(Fd, Chunk),
+    ok = couch_file:write_header(Fd, header),
+    couch_file:close(Fd),
+
+    {ok, SneakyFd} = file:open(Path, [binary, read, write, raw]),
+    ok = file:pwrite(SneakyFd, Pos + 100, <<"oops!">>),
+    file:close(SneakyFd),
+
+    {ok, Fd1} = couch_file:open(Path, []),
+    {ok, Header} = couch_file:read_header(Fd1),
+    ?assertEqual(header, Header),
+    ?assertExit({file_corruption, <<"file corruption">>}, couch_file:pread_binary(Fd1, Pos)),
+    catch couch_file:close(Fd1).
+
+t_can_detect_block_corruption_with_legacy_checksum({_Ctx, Path}) ->
+    disable_xxhash(),
+
+    {ok, Fd} = couch_file:open(Path, [create]),
+    Bin = crypto:strong_rand_bytes(100000),
+    Chunk = couch_file:assemble_file_chunk_and_checksum(Bin),
+    {ok, Pos, _} = couch_file:append_raw_chunk(Fd, Chunk),
+    ok = couch_file:write_header(Fd, header),
+    couch_file:close(Fd),
+
+    {ok, SneakyFd} = file:open(Path, [write, binary, read, raw]),
+    ok = file:pwrite(SneakyFd, Pos + 100, <<"oops!">>),
+    file:close(SneakyFd),
+
+    {ok, Fd1} = couch_file:open(Path, []),
+    {ok, Header} = couch_file:read_header(Fd1),
+    ?assertEqual(header, Header),
+    ?assertExit({file_corruption, <<"file corruption">>}, couch_file:pread_binary(Fd1, Pos)),
+    catch couch_file:close(Fd1).
+
+enable_xxhash() ->
+    couch_file:reset_checksum_persistent_term_config(),
+    reset_legacy_checksum_stats(),
+    config:set("couchdb", "write_xxhash_checksums", "true", _Persist = false).
+
+disable_xxhash() ->
+    couch_file:reset_checksum_persistent_term_config(),
+    reset_legacy_checksum_stats(),
+    config:set("couchdb", "write_xxhash_checksums", "false", _Persist = false).
+
+legacy_stats() ->
+    couch_stats:sample([couchdb, legacy_checksums]).
+
+reset_legacy_checksum_stats() ->
+    Counter = couch_stats:sample([couchdb, legacy_checksums]),
+    couch_stats:decrement_counter([couchdb, legacy_checksums], Counter).
