@@ -28,7 +28,6 @@
     db_close/1,
     get_db_info/1,
     get_pending_count/2,
-    get_view_info/3,
     update_doc/3,
     update_doc/4,
     update_docs/3,
@@ -154,17 +153,6 @@ get_pending_count(#httpdb{} = Db, Seq) ->
         {ok, couch_util:get_value(<<"pending">>, Props, null)}
     end).
 
-get_view_info(#httpdb{} = Db, DDocId, ViewName) ->
-    Path = io_lib:format("~s/_view/~s/_info", [DDocId, ViewName]),
-    send_req(
-        Db,
-        [{path, Path}],
-        fun(200, _, {Props}) ->
-            {VInfo} = couch_util:get_value(<<"view_index">>, Props, {[]}),
-            {ok, VInfo}
-        end
-    ).
-
 ensure_full_commit(#httpdb{} = Db) ->
     send_req(
         Db,
@@ -277,14 +265,6 @@ bulk_get_zip({Id, Rev, _}, {[_ | _] = Props}) ->
     end.
 
 -spec open_doc_revs(#httpdb{}, binary(), list(), list(), function(), any()) -> no_return().
-open_doc_revs(#httpdb{retries = 0} = HttpDb, Id, Revs, Options, _Fun, _Acc) ->
-    Path = encode_doc_id(Id),
-    QS = options_to_query_args(HttpDb, Path, [revs, {open_revs, Revs} | Options]),
-    Url = couch_util:url_strip_password(
-        couch_replicator_httpc:full_url(HttpDb, [{path, Path}, {qs, QS}])
-    ),
-    couch_log:error("Replication crashing because GET ~s failed", [Url]),
-    exit(kaboom);
 open_doc_revs(#httpdb{} = HttpDb, Id, Revs, Options, Fun, Acc) ->
     Path = encode_doc_id(Id),
     QS = options_to_query_args(HttpDb, Path, [revs, {open_revs, Revs} | Options]),
@@ -343,6 +323,8 @@ open_doc_revs(#httpdb{} = HttpDb, Id, Revs, Options, Fun, Acc) ->
             throw(Stub);
         {'DOWN', Ref, process, Pid, {http_request_failed, _, _, max_backoff}} ->
             exit(max_backoff);
+        {'DOWN', Ref, process, Pid, {doc_write_failed, _} = Error} ->
+            exit(Error);
         {'DOWN', Ref, process, Pid, request_uri_too_long} ->
             NewMaxLen = get_value(max_url_len, Options, ?MAX_URL_LEN) div 2,
             case NewMaxLen < ?MIN_URL_LEN of
@@ -367,17 +349,20 @@ open_doc_revs(#httpdb{} = HttpDb, Id, Revs, Options, Fun, Acc) ->
                 couch_replicator_httpc:full_url(HttpDb, [{path, Path}, {qs, QS}])
             ),
             #httpdb{retries = Retries, wait = Wait0} = HttpDb,
-            Wait = 2 * erlang:min(Wait0 * 2, ?MAX_WAIT),
-            couch_log:notice(
-                "Retrying GET to ~s in ~p seconds due to error ~w",
-                [Url, Wait / 1000, error_reason(Else)]
-            ),
-            ok = timer:sleep(Wait),
-            RetryDb = HttpDb#httpdb{
-                retries = Retries - 1,
-                wait = Wait
-            },
-            open_doc_revs(RetryDb, Id, Revs, Options, Fun, Acc)
+            NewRetries = Retries - 1,
+            case NewRetries > 0 of
+                true ->
+                    Wait = 2 * erlang:min(Wait0 * 2, ?MAX_WAIT),
+                    LogRetryMsg = "Retrying GET to ~s in ~p seconds due to error ~w",
+                    couch_log:notice(LogRetryMsg, [Url, Wait / 1000, error_reason(Else)]),
+                    ok = timer:sleep(Wait),
+                    RetryDb = HttpDb#httpdb{retries = NewRetries, wait = Wait},
+                    open_doc_revs(RetryDb, Id, Revs, Options, Fun, Acc);
+                false ->
+                    LogFailMsg = "Replication crashing because GET ~s failed. Error ~p",
+                    couch_log:error(LogFailMsg, [Url, error_reason(Else)]),
+                    exit(open_doc_revs_failed)
+            end
     end.
 
 error_reason({http_request_failed, "GET", _Url, {error, timeout}}) ->
@@ -451,17 +436,25 @@ update_doc(#httpdb{} = HttpDb, #doc{id = DocId} = Doc, Options, Type) ->
             (409, _, _) ->
                 throw(conflict);
             (Code, _, {Props}) ->
-                case {Code, get_value(<<"error">>, Props)} of
-                    {401, <<"unauthorized">>} ->
-                        throw({unauthorized, get_value(<<"reason">>, Props)});
-                    {403, <<"forbidden">>} ->
-                        throw({forbidden, get_value(<<"reason">>, Props)});
-                    {412, <<"missing_stub">>} ->
-                        throw({missing_stub, get_value(<<"reason">>, Props)});
-                    {413, _} ->
+                Error = get_value(<<"error">>, Props),
+                Reason = get_value(<<"reason">>, Props),
+                case {Code, Error, Reason} of
+                    {401, <<"unauthorized">>, _} ->
+                        throw({unauthorized, Reason});
+                    {403, <<"forbidden">>, _} ->
+                        throw({forbidden, Reason});
+                    {412, <<"missing_stub">>, _} ->
+                        throw({missing_stub, Reason});
+                    {413, _, _} ->
                         {error, request_body_too_large};
-                    {_, Error} ->
-                        {error, Error}
+                    {415, _, _} ->
+                        {error, unsupported_media_type};
+                    {400, <<"bad_request">>, <<"Attachment name ", _/binary>>} ->
+                        {error, {invalid_attachment_name, Reason}};
+                    {_, undefined, _} ->
+                        {error, {Code, Props}};
+                    {_, _, _} ->
+                        {error, {Error, Reason}}
                 end
         end
     ).
