@@ -25,7 +25,6 @@
     handle_call/3,
     handle_info/2,
     handle_cast/2,
-    code_change/3,
     format_status/2
 ]).
 
@@ -41,7 +40,8 @@
     jobs/0,
     job/1,
     restart_job/1,
-    update_job_stats/2
+    update_job_stats/2,
+    get_interval_msec/0
 ]).
 
 %% config_listener callbacks
@@ -178,13 +178,13 @@ health_threshold() ->
         ?DEFAULT_HEALTH_THRESHOLD_SEC
     ).
 
--spec find_jobs_by_dbname(binary()) -> list(#rep{}).
+-spec find_jobs_by_dbname(binary()) -> list(rep_id()).
 find_jobs_by_dbname(DbName) ->
     Rep = #rep{db_name = DbName, _ = '_'},
     MatchSpec = #job{id = '$1', rep = Rep, _ = '_'},
     [RepId || [RepId] <- ets:match(?MODULE, MatchSpec)].
 
--spec find_jobs_by_doc(binary(), binary()) -> list(#rep{}).
+-spec find_jobs_by_doc(binary(), binary()) -> list(rep_id()).
 find_jobs_by_doc(DbName, DocId) ->
     Rep = #rep{db_name = DbName, doc_id = DocId, _ = '_'},
     MatchSpec = #job{id = '$1', rep = Rep, _ = '_'},
@@ -206,9 +206,13 @@ restart_job(JobId) ->
 update_job_stats(JobId, Stats) ->
     gen_server:cast(?MODULE, {update_job_stats, JobId, Stats}).
 
+get_interval_msec() ->
+    config:get_integer("replicator", "interval", ?DEFAULT_SCHEDULER_INTERVAL).
+
 %% gen_server functions
 
 init(_) ->
+    process_flag(trap_exit, true),
     config:enable_feature('scheduler'),
     EtsOpts = [
         named_table,
@@ -219,11 +223,7 @@ init(_) ->
     ?MODULE = ets:new(?MODULE, EtsOpts),
     ok = couch_replicator_share:init(),
     ok = config:listen_for_changes(?MODULE, nil),
-    Interval = config:get_integer(
-        "replicator",
-        "interval",
-        ?DEFAULT_SCHEDULER_INTERVAL
-    ),
+    Interval = get_interval_msec(),
     MaxJobs = config:get_integer("replicator", "max_jobs", ?DEFAULT_MAX_JOBS),
     MaxChurn = config:get_integer(
         "replicator",
@@ -317,7 +317,10 @@ handle_info(reschedule, State) ->
     erlang:cancel_timer(State#state.timer),
     Timer = erlang:send_after(State#state.interval, self(), reschedule),
     {noreply, State#state{timer = Timer}};
-handle_info({'DOWN', _Ref, process, Pid, normal}, State) ->
+handle_info({'EXIT', Pid, Reason}, #state{stats_pid = Pid} = State) ->
+    couch_log:error("~p : stats updater died: ~p", [?MODULE, Reason]),
+    {stop, {status_updater_died, Reason}, State};
+handle_info({'EXIT', Pid, normal}, State) ->
     {ok, Job} = job_by_pid(Pid),
     couch_log:notice("~p: Job ~p completed normally", [?MODULE, Job#job.id]),
     Interval = State#state.interval,
@@ -325,7 +328,7 @@ handle_info({'DOWN', _Ref, process, Pid, normal}, State) ->
     remove_job_int(Job),
     update_running_jobs_stats(State#state.stats_pid),
     {noreply, State};
-handle_info({'DOWN', _Ref, process, Pid, Reason0}, State) ->
+handle_info({'EXIT', Pid, Reason0}, State) ->
     {ok, Job} = job_by_pid(Pid),
     Reason =
         case Reason0 of
@@ -342,9 +345,6 @@ handle_info(restart_config_listener, State) ->
 handle_info(_, State) ->
     {noreply, State}.
 
-code_change(_OldVsn, State, _Extra) ->
-    {ok, State}.
-
 terminate(_Reason, _State) ->
     couch_replicator_share:clear(),
     ok.
@@ -627,29 +627,18 @@ start_job_int(#job{pid = Pid}, _State) when Pid /= undefined ->
     ok;
 start_job_int(#job{} = Job0, State) ->
     Job = maybe_optimize_job_for_rate_limiting(Job0),
-    case couch_replicator_scheduler_sup:start_child(Job#job.rep) of
+    case couch_replicator_scheduler_job:start_link(Job#job.rep) of
         {ok, Child} ->
-            Ref = monitor(process, Child),
-            ok = update_state_started(Job, Child, Ref, State),
+            ok = update_state_started(Job, Child, State),
             couch_log:notice(
                 "~p: Job ~p started as ~p",
                 [?MODULE, Job#job.id, Child]
             );
-        {error, {already_started, OtherPid}} when node(OtherPid) =:= node() ->
-            Ref = monitor(process, OtherPid),
-            ok = update_state_started(Job, OtherPid, Ref, State),
-            couch_log:notice(
-                "~p: Job ~p already running as ~p. Most likely"
-                " because replicator scheduler was restarted",
-                [?MODULE, Job#job.id, OtherPid]
-            );
-        {error, {already_started, OtherPid}} when node(OtherPid) =/= node() ->
-            CrashMsg = "Duplicate replication running on another node",
-            couch_log:notice(
-                "~p: Job ~p already running as ~p. Most likely"
-                " because a duplicate replication is running on another node",
-                [?MODULE, Job#job.id, OtherPid]
-            ),
+        {error, {already_started, OtherPid}} ->
+            Node = node(OtherPid),
+            CrashMsg = "Duplicate replication running on " ++ atom_to_list(Node),
+            LogMsg = "~p: Job ~p already running as ~p on node ~s",
+            couch_log:warning(LogMsg, [?MODULE, Job#job.id, OtherPid, Node]),
             ok = update_state_crashed(Job, CrashMsg, State);
         {error, Reason} ->
             couch_log:notice(
@@ -662,14 +651,10 @@ start_job_int(#job{} = Job0, State) ->
 -spec stop_job_int(#job{}, #state{}) -> ok | {error, term()}.
 stop_job_int(#job{pid = undefined}, _State) ->
     ok;
-stop_job_int(#job{} = Job, State) ->
-    ok = couch_replicator_scheduler_sup:terminate_child(Job#job.pid),
-    demonitor(Job#job.monitor, [flush]),
+stop_job_int(#job{pid = Pid} = Job, State) when is_pid(Pid) ->
+    ok = couch_replicator_scheduler_job:stop(Pid),
     ok = update_state_stopped(Job, State),
-    couch_log:notice(
-        "~p: Job ~p stopped as ~p",
-        [?MODULE, Job#job.id, Job#job.pid]
-    ).
+    couch_log:notice("~p: Job ~p stopped as ~p", [?MODULE, Job#job.id, Pid]).
 
 -spec remove_job_int(#job{}) -> true.
 remove_job_int(#job{} = Job) ->
@@ -708,15 +693,15 @@ job_by_id(Id) ->
 
 -spec update_state_stopped(#job{}, #state{}) -> ok.
 update_state_stopped(Job, State) ->
-    Job1 = reset_job_process(Job),
+    Job1 = Job#job{pid = undefined},
     Job2 = update_history(Job1, stopped, os:timestamp(), State),
     true = ets:insert(?MODULE, Job2),
     couch_stats:increment_counter([couch_replicator, jobs, stops]),
     ok.
 
--spec update_state_started(#job{}, pid(), reference(), #state{}) -> ok.
-update_state_started(Job, Pid, Ref, State) ->
-    Job1 = set_job_process(Job, Pid, Ref),
+-spec update_state_started(#job{}, pid(), #state{}) -> ok.
+update_state_started(Job, Pid, State) when is_pid(Pid) ->
+    Job1 = Job#job{pid = Pid},
     Job2 = update_history(Job1, started, os:timestamp(), State),
     true = ets:insert(?MODULE, Job2),
     couch_stats:increment_counter([couch_replicator, jobs, starts]),
@@ -724,20 +709,12 @@ update_state_started(Job, Pid, Ref, State) ->
 
 -spec update_state_crashed(#job{}, any(), #state{}) -> ok.
 update_state_crashed(Job, Reason, State) ->
-    Job1 = reset_job_process(Job),
+    Job1 = Job#job{pid = undefined},
     Job2 = update_history(Job1, {crashed, Reason}, os:timestamp(), State),
     true = ets:insert(?MODULE, Job2),
     couch_stats:increment_counter([couch_replicator, jobs, crashes]),
     ok.
 
--spec set_job_process(#job{}, pid(), reference()) -> #job{}.
-set_job_process(#job{} = Job, Pid, Ref) when is_pid(Pid), is_reference(Ref) ->
-    Job#job{pid = Pid, monitor = Ref}.
-
--spec reset_job_process(#job{}) -> #job{}.
-reset_job_process(#job{} = Job) ->
-    Job#job{pid = undefined, monitor = undefined}.
-
 -spec reschedule(#state{}) -> ok.
 reschedule(#state{interval = Interval} = State) ->
     couch_replicator_share:update(running_jobs(), Interval, os:timestamp()),
@@ -1257,7 +1234,7 @@ t_jobs_dont_churn_if_there_are_available_running_slots() ->
         reschedule(mock_state(2, 2)),
         ?assertEqual({2, 0}, run_stop_count()),
         ?assertEqual([], jobs_stopped()),
-        ?assertEqual(0, meck:num_calls(couch_replicator_scheduler_sup, start_child, 1))
+        ?assertEqual(0, meck:num_calls(couch_replicator_scheduler_job, start_link, 1))
     end).
 
 t_start_only_pending_jobs_do_not_churn_existing_ones() ->
@@ -1267,7 +1244,7 @@ t_start_only_pending_jobs_do_not_churn_existing_ones() ->
             continuous_running(2)
         ]),
         reschedule(mock_state(2, 2)),
-        ?assertEqual(1, meck:num_calls(couch_replicator_scheduler_sup, start_child, 1)),
+        ?assertEqual(1, meck:num_calls(couch_replicator_scheduler_job, start_link, 1)),
         ?assertEqual([], jobs_stopped()),
         ?assertEqual({2, 0}, run_stop_count())
     end).
@@ -1383,7 +1360,7 @@ t_if_transient_job_crashes_it_gets_removed() ->
         ?assertEqual(1, ets:info(?MODULE, size)),
         State = #state{max_history = 3, stats_pid = self()},
         {noreply, State} = handle_info(
-            {'DOWN', r1, process, Pid, failed},
+            {'EXIT', Pid, failed},
             State
         ),
         ?assertEqual(0, ets:info(?MODULE, size))
@@ -1407,7 +1384,7 @@ t_if_permanent_job_crashes_it_stays_in_ets() ->
             stats_pid = self()
         },
         {noreply, State} = handle_info(
-            {'DOWN', r1, process, Pid, failed},
+            {'EXIT', Pid, failed},
             State
         ),
         ?assertEqual(1, ets:info(?MODULE, size)),
@@ -1536,11 +1513,11 @@ setup_all() ->
     meck:expect(couch_log, notice, 2, ok),
     meck:expect(couch_log, warning, 2, ok),
     meck:expect(couch_log, error, 2, ok),
-    meck:expect(couch_replicator_scheduler_sup, terminate_child, 1, ok),
+    meck:expect(couch_replicator_scheduler_job, stop, 1, ok),
     meck:expect(couch_stats, increment_counter, 1, ok),
     meck:expect(couch_stats, update_gauge, 2, ok),
     Pid = mock_pid(),
-    meck:expect(couch_replicator_scheduler_sup, start_child, 1, {ok, Pid}),
+    meck:expect(couch_replicator_scheduler_job, start_link, 1, {ok, Pid}),
     couch_replicator_share:init().
 
 teardown_all(_) ->
@@ -1551,7 +1528,7 @@ teardown_all(_) ->
 setup() ->
     meck:reset([
         couch_log,
-        couch_replicator_scheduler_sup,
+        couch_replicator_scheduler_job,
         couch_stats,
         config
     ]).
@@ -1636,8 +1613,7 @@ continuous_running(Id) when is_integer(Id) ->
         id = Id,
         history = [started(Started), added()],
         rep = continuous_rep(),
-        pid = Pid,
-        monitor = monitor(process, Pid)
+        pid = Pid
     }.
 
 oneshot(Id) when is_integer(Id) ->
@@ -1652,8 +1628,7 @@ oneshot_running(Id) when is_integer(Id) ->
         id = Id,
         history = [started(Started), added()],
         rep = rep(),
-        pid = Pid,
-        monitor = monitor(process, Pid)
+        pid = Pid
     }.
 
 testjob(Hist) when is_list(Hist) ->
