@@ -16,9 +16,10 @@ import Components from '../components/react-components';
 import NewReplication from './components/newreplication';
 import Activity from './components/activity';
 import {checkReplicationDocID} from './api';
-import {OnePane, OnePaneHeader, OnePaneContent} from '../components/layouts';
+import {OnePane, OnePaneHeader, OnePaneContent, OnePaneFooter} from '../components/layouts';
 import {TabElementWrapper, TabElement} from '../components/components/tabelement';
 import ReplicateActivity from './components/replicate-activity';
+import { ReplicationFooter } from './components/replication-footer';
 
 const {LoadLines, Polling, RefreshBtn} = Components;
 
@@ -38,7 +39,7 @@ export default class ReplicationController extends React.Component {
   }
 
   getAllActivity () {
-    this.props.getReplicationActivity();
+    this.props.getReplicationActivity(this.props.pageLimit);
     this.props.getReplicateActivity();
   }
 
@@ -70,7 +71,8 @@ export default class ReplicationController extends React.Component {
       hideConflictModal, isConflictModalVisible, filterDocs,
       filterReplicate, replicate, clearReplicationForm, selectAllDocs, changeActivitySort, selectDoc,
       deleteDocs, deleteReplicates, selectAllReplicates, selectReplicate,
-      sourceAuthType, sourceAuth, targetAuthType, targetAuth, targetDatabasePartitioned, allowNewPartitionedLocalDbs
+      sourceAuthType, sourceAuth, targetAuthType, targetAuth, targetDatabasePartitioned, allowNewPartitionedLocalDbs,
+      pageLimit
     } = this.props;
 
     if (tabSection === 'new replication') {
@@ -106,6 +108,7 @@ export default class ReplicationController extends React.Component {
         checkReplicationDocID={checkReplicationDocID}
         authenticated={authenticated}
         submittedNoChange={submittedNoChange}
+        pageLimit={pageLimit}
       />;
     }
 
@@ -143,6 +146,7 @@ export default class ReplicationController extends React.Component {
       deleteDocs={deleteDocs}
       activitySort={activitySort}
       changeActivitySort={changeActivitySort}
+      pageLimit={pageLimit}
     />;
   }
 
@@ -220,6 +224,21 @@ export default class ReplicationController extends React.Component {
     );
   }
 
+  getFooter () {
+    const { tabSection, statusDocs, pageLimit, setPageLimit } = this.props;
+
+    if (tabSection === 'activity') {
+      return (
+        <ReplicationFooter
+          statusDocs={statusDocs}
+          pageLimit={pageLimit}
+          setPageLimit={setPageLimit}
+        />
+      );
+    }
+    return null;
+  }
+
   render () {
     const { checkingAPI } = this.props;
 
@@ -235,11 +254,14 @@ export default class ReplicationController extends React.Component {
         <OnePaneContent>
           <div className="template-content flex-body flex-layout flex-col">
             {this.getTabs()}
-            <div className="replication__page flex-layout flex-col">
+            <div className="replication__page">
               {this.showSection()}
             </div>
           </div>
         </OnePaneContent>
+        <OnePaneFooter>
+          {this.getFooter()}
+        </OnePaneFooter>
       </OnePane>
     );
   }
