@@ -12,7 +12,6 @@
 
 -module(global_changes_server).
 -behaviour(gen_server).
--vsn(1).
 
 -export([
     start_link/0
@@ -20,11 +19,9 @@
 
 -export([
     init/1,
-    terminate/2,
     handle_call/3,
     handle_cast/2,
-    handle_info/2,
-    code_change/3
+    handle_info/2
 ]).
 
 -export([
@@ -69,16 +66,13 @@ init([]) ->
     State = #state{
         update_db = UpdateDb,
         pending_update_count = 0,
-        pending_updates = sets:new(),
+        pending_updates = sets:new([{version, 2}]),
         max_write_delay = MaxWriteDelay,
         dbname = GlobalChangesDbName,
         handler_ref = erlang:monitor(process, Handler)
     },
     {ok, State}.
 
-terminate(_Reason, _Srv) ->
-    ok.
-
 handle_call(_Msg, _From, State) ->
     {reply, ok, State}.
 
@@ -106,7 +100,7 @@ handle_cast({set_update_db, Boolean}, State0) ->
             {false, true} ->
                 State0#state{
                     update_db = Boolean,
-                    pending_updates = sets:new(),
+                    pending_updates = sets:new([{version, 2}]),
                     pending_update_count = 0
                 };
             _ ->
@@ -138,9 +132,6 @@ handle_info({'DOWN', Ref, _, _, Reason}, #state{handler_ref = Ref} = State) ->
 handle_info(_, State) ->
     {noreply, State}.
 
-code_change(_OldVsn, State, _Extra) ->
-    {ok, State}.
-
 flush_updates(State) ->
     DocIds = sets:to_list(State#state.pending_updates),
     try group_ids_by_shard(State#state.dbname, DocIds) of
@@ -177,7 +168,7 @@ flush_updates(State) ->
         0
     ),
     {noreply, State#state{
-        pending_updates = sets:new(),
+        pending_updates = sets:new([{version, 2}]),
         pending_update_count = 0
     }}.
 
