@@ -5,8 +5,8 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "erl_nif.h"
 #include "jiffy.h"
+#include "termstack.h"
 
 #define BIN_INC_SIZE 2048
 
@@ -39,15 +39,14 @@ typedef struct {
     int             shiftcnt;
     int             count;
 
-    size_t          iolen;
     size_t          iosize;
     ERL_NIF_TERM    iolist;
-    ErlNifBinary    bin;
-    ErlNifBinary*   curr;
+    int             partial_output;
 
+    ErlNifBinary    buffer;
+    int             have_buffer;
 
-    char*           p;
-    unsigned char*  u;
+    unsigned char*  p;
     size_t          i;
 } Encoder;
 
@@ -82,19 +81,19 @@ enc_new(ErlNifEnv* env)
     e->shiftcnt = 0;
     e->count = 0;
 
-    e->iolen = 0;
     e->iosize = 0;
-    e->curr = &(e->bin);
-    if(!enif_alloc_binary(BIN_INC_SIZE, e->curr)) {
-        e->curr = NULL;
+    e->iolist = enif_make_list(env, 0);
+
+    e->partial_output = 0;
+
+    if(!enif_alloc_binary(BIN_INC_SIZE, &e->buffer)) {
         enif_release_resource(e);
         return NULL;
     }
 
-    memset(e->curr->data, 0, e->curr->size);
+    e->have_buffer = 1;
 
-    e->p = (char*) e->curr->data;
-    e->u = (unsigned char*) e->curr->data;
+    e->p = e->buffer.data;
     e->i = 0;
 
     return e;
@@ -112,8 +111,8 @@ enc_destroy(ErlNifEnv* env, void* obj)
 {
     Encoder* e = (Encoder*) obj;
 
-    if(e->curr != NULL) {
-        enif_release_binary(e->curr);
+    if(e->have_buffer) {
+        enif_release_binary(&e->buffer);
     }
 }
 
@@ -130,100 +129,60 @@ enc_obj_error(Encoder* e, const char* msg, ERL_NIF_TERM obj)
     return make_obj_error(e->atoms, e->env, msg, obj);
 }
 
-static inline int
-enc_ensure(Encoder* e, size_t req)
-{
-    size_t need = e->curr->size;
-    while(req >= (need - e->i)) need <<= 1;
-
-    if(need != e->curr->size) {
-        if(!enif_realloc_binary(e->curr, need)) {
-            return 0;
-        }
-        e->p = (char*) e->curr->data;
-        e->u = (unsigned char*) e->curr->data;
-    }
-
-    return 1;
-}
-
 int
-enc_result(Encoder* e, ERL_NIF_TERM* value)
+enc_flush(Encoder* e)
 {
-    if(e->i != e->curr->size) {
-        if(!enif_realloc_binary(e->curr, e->i)) {
+    ERL_NIF_TERM bin;
+
+    if(e->i == 0) {
+        return 1;
+    }
+
+    if(e->i < e->buffer.size) {
+        if(!enif_realloc_binary(&e->buffer, e->i)) {
             return 0;
         }
     }
 
-    *value = enif_make_binary(e->env, e->curr);
-    e->curr = NULL;
-    return 1;
-}
+    bin = enif_make_binary(e->env, &e->buffer);
+    e->have_buffer = 0;
 
-int
-enc_done(Encoder* e, ERL_NIF_TERM* value)
-{
-    ERL_NIF_TERM last;
+    e->iolist = enif_make_list_cell(e->env, bin, e->iolist);
+    e->iosize += e->i;
 
-    if(e->iolen == 0) {
-        return enc_result(e, value);
-    }
-
-    if(e->i > 0 ) {
-        if(!enc_result(e, &last)) {
-            return 0;
-        }
-
-        e->iolist = enif_make_list_cell(e->env, last, e->iolist);
-        e->iolen++;
-    }
-
-    *value = e->iolist;
     return 1;
 }
 
 static inline int
-enc_unknown(Encoder* e, ERL_NIF_TERM value)
+enc_ensure(Encoder* e, size_t req)
 {
-    ErlNifBinary* bin = e->curr;
-    ERL_NIF_TERM curr;
+    size_t new_size = BIN_INC_SIZE;
 
-    if(e->i > 0) {
-        if(!enc_result(e, &curr)) {
+    if(e->have_buffer) {
+        if(req < (e->buffer.size - e->i)) {
+            return 1;
+        }
+
+        if(!enc_flush(e)) {
             return 0;
         }
 
-        e->iolist = enif_make_list_cell(e->env, curr, e->iolist);
-        e->iolen++;
-    }
-
-    e->iolist = enif_make_list_cell(e->env, value, e->iolist);
-    e->iolen++;
-
-    // Track the total number of bytes produced before
-    // splitting our IO buffer. We add 16 to this value
-    // as a rough estimate of the number of bytes that
-    // a bignum might produce when encoded.
-    e->iosize += e->i + 16;
-
-    // Reinitialize our binary for the next buffer if we
-    // used any data in the buffer. If we haven't used any
-    // bytes in the buffer then we can safely reuse it
-    // for anything following the unknown value.
-    if(e->i > 0) {
-        e->curr = bin;
-        if(!enif_alloc_binary(BIN_INC_SIZE, e->curr)) {
-            return 0;
+        if(e->have_buffer) {
+            return 1;
         }
-
-        memset(e->curr->data, 0, e->curr->size);
-
-        e->p = (char*) e->curr->data;
-        e->u = (unsigned char*) e->curr->data;
-        e->i = 0;
     }
 
+    for(new_size = BIN_INC_SIZE; new_size < req; new_size <<= 1);
+
+    if(!enif_alloc_binary(new_size, &e->buffer)) {
+        return 0;
+    }
+
+    e->have_buffer = 1;
+
+    e->p = e->buffer.data;
+    e->i = 0;
+
     return 1;
 }
 
@@ -241,79 +200,80 @@ enc_literal(Encoder* e, const char* literal, size_t len)
 }
 
 static inline int
-enc_string(Encoder* e, ERL_NIF_TERM val)
-{
-    ErlNifBinary bin;
-    char atom[512];
-
-    unsigned char* data;
-    size_t size;
-
-    int esc_extra = 0;
-    int ulen;
-    int uval;
-    int i;
-
-    if(enif_is_binary(e->env, val)) {
-        if(!enif_inspect_binary(e->env, val, &bin)) {
-            return 0;
-        }
-        data = bin.data;
-        size = bin.size;
-    } else if(enif_is_atom(e->env, val)) {
-        if(!enif_get_atom(e->env, val, atom, 512, ERL_NIF_LATIN1)) {
-            return 0;
-        }
-        data = (unsigned char*) atom;
-        size = strlen(atom);
-    } else {
+enc_unknown(Encoder* e, ERL_NIF_TERM value) {
+    // Bignums are encoded in Erlang as the NIF API
+    // does not have functions for dealing with them.
+    if(!enc_flush(e)) {
         return 0;
     }
 
-    i = 0;
-    while(i < size) {
-        switch((char) data[i]) {
-            case '\"':
-            case '\\':
-            case '\b':
-            case '\f':
-            case '\n':
-            case '\r':
-            case '\t':
-                esc_extra += 1;
-                i++;
-                continue;
-            case '/':
-                if(e->escape_forward_slashes) {
-                    esc_extra += 1;
-                    i++;
-                    continue;
-                }
-            default:
-                if(data[i] < 0x20) {
-                    esc_extra += 5;
-                    i++;
-                    continue;
-                } else if(data[i] < 0x80) {
-                    i++;
-                    continue;
-                }
-                ulen = utf8_validate(&(data[i]), size - i);
-                if(ulen < 0) {
-                    return 0;
-                }
-                if(e->uescape) {
-                    uval = utf8_to_unicode(&(data[i]), ulen);
-                    if(uval < 0) {
-                        return 0;
-                    }
-                    esc_extra += utf8_esc_len(uval) - ulen;
-                }
-                i += ulen;
-        }
+    e->iolist = enif_make_list_cell(e->env, value, e->iolist);
+    e->partial_output = 1;
+
+    return 1;
+}
+
+static inline int
+enc_special_character(Encoder* e, int val) {
+    switch(val) {
+        case '\"':
+        case '\\':
+            e->p[e->i++] = '\\';
+            e->p[e->i++] = val;
+            return 1;
+        case '\b':
+            e->p[e->i++] = '\\';
+            e->p[e->i++] = 'b';
+            return 1;
+        case '\f':
+            e->p[e->i++] = '\\';
+            e->p[e->i++] = 'f';
+            return 1;
+        case '\n':
+            e->p[e->i++] = '\\';
+            e->p[e->i++] = 'n';
+            return 1;
+        case '\r':
+            e->p[e->i++] = '\\';
+            e->p[e->i++] = 'r';
+            return 1;
+        case '\t':
+            e->p[e->i++] = '\\';
+            e->p[e->i++] = 't';
+            return 1;
+        case '/':
+            if(e->escape_forward_slashes) {
+                e->p[e->i++] = '\\';
+            }
+            e->p[e->i++] = '/';
+            return 1;
+        default:
+            if(val < 0x20) {
+                e->i += unicode_uescape(val, &(e->p[e->i]));
+                return 1;
+            }
+
+            return 0;
+    }
+}
+
+static int
+enc_atom(Encoder* e, ERL_NIF_TERM val)
+{
+    static const int MAX_ESCAPE_LEN = 12;
+    unsigned char data[512];
+
+    size_t size;
+    int i;
+
+    if(!enif_get_atom(e->env, val, (char*)data, 512, ERL_NIF_LATIN1)) {
+        return 0;
     }
 
-    if(!enc_ensure(e, size + esc_extra + 2)) {
+    size = strlen((const char*)data);
+
+    /* Reserve space for the first quotation mark and most of the output. */
+    if(!enc_ensure(e, size + MAX_ESCAPE_LEN + 1)) {
         return 0;
     }
 
@@ -321,74 +281,30 @@ enc_string(Encoder* e, ERL_NIF_TERM val)
 
     i = 0;
     while(i < size) {
-        switch((char) data[i]) {
-            case '\"':
-            case '\\':
-                e->p[e->i++] = '\\';
-                e->u[e->i++] = data[i];
-                i++;
-                continue;
-            case '\b':
-                e->p[e->i++] = '\\';
-                e->p[e->i++] = 'b';
-                i++;
-                continue;
-            case '\f':
-                e->p[e->i++] = '\\';
-                e->p[e->i++] = 'f';
-                i++;
-                continue;
-            case '\n':
-                e->p[e->i++] = '\\';
-                e->p[e->i++] = 'n';
-                i++;
-                continue;
-            case '\r':
-                e->p[e->i++] = '\\';
-                e->p[e->i++] = 'r';
-                i++;
-                continue;
-            case '\t':
-                e->p[e->i++] = '\\';
-                e->p[e->i++] = 't';
-                i++;
-                continue;
-            case '/':
-                if(e->escape_forward_slashes) {
-                    e->p[e->i++] = '\\';
-                    e->u[e->i++] = data[i];
-                    i++;
-                    continue;
-                }
-            default:
-                if(data[i] < 0x20) {
-                    ulen = unicode_uescape(data[i], &(e->p[e->i]));
-                    if(ulen < 0) {
-                        return 0;
-                    }
-                    e->i += ulen;
-                    i++;
-                } else if((data[i] & 0x80) && e->uescape) {
-                    uval = utf8_to_unicode(&(data[i]), size-i);
-                    if(uval < 0) {
-                        return 0;
-                    }
-
-                    ulen = unicode_uescape(uval, &(e->p[e->i]));
-                    if(ulen < 0) {
-                        return 0;
-                    }
-                    e->i += ulen;
-
-                    ulen = utf8_len(uval);
-                    if(ulen < 0) {
-                        return 0;
-                    }
-                    i += ulen;
-                } else {
-                    e->u[e->i++] = data[i++];
-                }
+        if(!enc_ensure(e, MAX_ESCAPE_LEN)) {
+            return 0;
         }
+
+        if(enc_special_character(e, data[i])) {
+            i++;
+        } else if(data[i] < 0x80) {
+            e->p[e->i++] = data[i];
+            i++;
+        } else if(data[i] >= 0x80) {
+            /* The atom encoding is latin1, so we don't need validation
+             * as all latin1 characters are valid Unicode codepoints. */
+            if (!e->uescape) {
+                e->i += unicode_to_utf8(data[i], &e->p[e->i]);
+            } else {
+                e->i += unicode_uescape(data[i], &e->p[e->i]);
+            }
+
+            i++;
+        }
+    }
+
+    if(!enc_ensure(e, 1)) {
+        return 0;
     }
 
     e->p[e->i++] = '\"';
@@ -397,6 +313,168 @@ enc_string(Encoder* e, ERL_NIF_TERM val)
     return 1;
 }
 
+static int
+enc_string(Encoder* e, ERL_NIF_TERM val)
+{
+    static const int MAX_ESCAPE_LEN = 12;
+    ErlNifBinary bin;
+
+    unsigned char* data;
+    size_t size;
+    int esc_len;
+    int ulen;
+    int uval;
+    int i;
+
+    if(!enif_inspect_binary(e->env, val, &bin)) {
+        return 0;
+    }
+
+    data = bin.data;
+    size = bin.size;
+
+    /* Reserve space for the first quotation mark and most of the output. */
+    if(!enc_ensure(e, size + MAX_ESCAPE_LEN + 1)) {
+        return 0;
+    }
+
+    e->p[e->i++] = '\"';
+
+    i = 0;
+    while(i < size) {
+        if(!enc_ensure(e, MAX_ESCAPE_LEN)) {
+            return 0;
+        }
+
+        if(enc_special_character(e, data[i])) {
+            i++;
+        } else if(data[i] < 0x80) {
+            e->p[e->i++] = data[i++];
+        } else if(data[i] >= 0x80) {
+            ulen = utf8_validate(&(data[i]), size - i);
+
+            if (ulen < 0) {
+                return 0;
+            } else if (e->uescape) {
+                uval = utf8_to_unicode(&(data[i]), size-i);
+                if(uval < 0) {
+                    return 0;
+                }
+
+                esc_len = unicode_uescape(uval, &(e->p[e->i]));
+                if(esc_len < 0) {
+                    return 0;
+                }
+
+                e->i += esc_len;
+            } else {
+                memcpy(&e->p[e->i], &data[i], ulen);
+                e->i += ulen;
+            }
+
+            i += ulen;
+        }
+    }
+
+    if(!enc_ensure(e, 1)) {
+        return 0;
+    }
+
+    e->p[e->i++] = '\"';
+    e->count++;
+
+    return 1;
+}
+
+static inline int
+enc_object_key(ErlNifEnv *env, Encoder* e, ERL_NIF_TERM val)
+{
+    if(enif_is_atom(env, val)) {
+        return enc_atom(e, val);
+    }
+
+    return enc_string(e, val);
+}
+
+// From https://www.slideshare.net/andreialexandrescu1/three-optimization-tips-for-c-15708507
+
+#define P01 10
+#define P02 100
+#define P03 1000
+#define P04 10000
+#define P05 100000
+#define P06 1000000
+#define P07 10000000
+#define P08 100000000
+#define P09 1000000000
+#define P10 10000000000
+#define P11 100000000000L
+#define P12 1000000000000L
+
+int
+digits10(ErlNifUInt64 v)
+{
+    if (v < P01) return 1;
+    if (v < P02) return 2;
+    if (v < P03) return 3;
+    if (v < P12) {
+        if (v < P08) {
+            if (v < P06) {
+                if (v < P04) {
+                    return 4;
+                }
+                return 5 + (v >= P05);
+            }
+            return 7 + (v >= P07);
+        }
+        if (v < P10) {
+            return 9 + (v >= P09);
+        }
+        return 11 + (v >= P11);
+    }
+    return 12 + digits10(v / P12);
+}
+
+unsigned int
+u64ToAsciiTable(unsigned char *dst, ErlNifUInt64 value)
+{
+    static const char digits[201] =
+        "0001020304050607080910111213141516171819"
+        "2021222324252627282930313233343536373839"
+        "4041424344454647484950515253545556575859"
+        "6061626364656667686970717273747576777879"
+        "8081828384858687888990919293949596979899";
+    const int length = digits10(value);
+    int next = length - 1;
+    while (value >= 100) {
+        const int i = (value % 100) * 2;
+        value /= 100;
+        dst[next] = digits[i + 1];
+        dst[next - 1] = digits[i];
+        next -= 2;
+    }
+    // Handle last 1-2 digits.
+    if (value < 10) {
+        dst[next] = '0' + (unsigned int) value;
+    } else {
+        const int i = (unsigned int) value * 2;
+        dst[next] = digits[i + 1];
+        dst[next - 1] = digits[i];
+    }
+    return length;
+}
+
+unsigned
+i64ToAsciiTable(unsigned char *dst, ErlNifSInt64 value)
+{
+    if (value < 0) {
+        *dst++ = '-';
+        return 1 + u64ToAsciiTable(dst, -value);
+    } else {
+        return u64ToAsciiTable(dst, value);
+    }
+}
+
 static inline int
 enc_long(Encoder* e, ErlNifSInt64 val)
 {
@@ -404,15 +482,7 @@ enc_long(Encoder* e, ErlNifSInt64 val)
         return 0;
     }
 
-#if (defined(__WIN32__) || defined(_WIN32) || defined(_WIN32_))
-    snprintf(&(e->p[e->i]), 32, "%lld", val);
-#elif SIZEOF_LONG == 8
-    snprintf(&(e->p[e->i]), 32, "%ld", val);
-#else
-    snprintf(&(e->p[e->i]), 32, "%lld", val);
-#endif
-
-    e->i += strlen(&(e->p[e->i]));
+    e->i += i64ToAsciiTable(&(e->p[e->i]), val);
     e->count++;
 
     return 1;
@@ -421,7 +491,7 @@ enc_long(Encoder* e, ErlNifSInt64 val)
 static inline int
 enc_double(Encoder* e, double val)
 {
-    char* start;
+    unsigned char* start;
     size_t len;
 
     if(!enc_ensure(e, 32)) {
@@ -430,7 +500,7 @@ enc_double(Encoder* e, double val)
 
     start = &(e->p[e->i]);
 
-    if(!double_to_shortest(start, e->curr->size, &len, val)) {
+    if(!double_to_shortest(start, e->buffer.size, &len, val)) {
         return 0;
     }
 
@@ -525,7 +595,6 @@ enc_comma(Encoder* e)
     return 1;
 }
 
-#if MAP_TYPE_PRESENT
 int
 enc_map_to_ejson(ErlNifEnv* env, ERL_NIF_TERM map, ERL_NIF_TERM* out)
 {
@@ -566,7 +635,6 @@ enc_map_to_ejson(ErlNifEnv* env, ERL_NIF_TERM map, ERL_NIF_TERM* out)
     *out = enif_make_tuple1(env, list);
     return 1;
 }
-#endif
 
 ERL_NIF_TERM
 encode_init(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
@@ -588,7 +656,7 @@ encode_init(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
     }
 
     tmp_argv[0] = enif_make_resource(env, e);
-    tmp_argv[1] = enif_make_list(env, 1, argv[0]);
+    tmp_argv[1] = enif_make_tuple1(env, argv[0]);
     tmp_argv[2] = enif_make_list(env, 0);
 
     enif_release_resource(e);
@@ -599,15 +667,15 @@ encode_init(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
     }
 
     while(enif_get_list_cell(env, opts, &val, &opts)) {
-        if(enif_compare(val, e->atoms->atom_uescape) == 0) {
+        if(enif_is_identical(val, e->atoms->atom_uescape)) {
             e->uescape = 1;
-        } else if(enif_compare(val, e->atoms->atom_pretty) == 0) {
+        } else if(enif_is_identical(val, e->atoms->atom_pretty)) {
             e->pretty = 1;
-        } else if(enif_compare(val, e->atoms->atom_escape_forward_slashes) == 0) {
+        } else if(enif_is_identical(val, e->atoms->atom_escape_forward_slashes)) {
             e->escape_forward_slashes = 1;
-        } else if(enif_compare(val, e->atoms->atom_use_nil) == 0) {
+        } else if(enif_is_identical(val, e->atoms->atom_use_nil)) {
             e->use_nil = 1;
-        } else if(enif_compare(val, e->atoms->atom_force_utf8) == 0) {
+        } else if(enif_is_identical(val, e->atoms->atom_force_utf8)) {
             // Ignore, handled in Erlang
         } else if(get_bytes_per_iter(env, val, &(e->bytes_per_red))) {
             continue;
@@ -624,147 +692,155 @@ encode_init(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
 ERL_NIF_TERM
 encode_iter(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
 {
-    Encoder* e;
     jiffy_st* st = (jiffy_st*) enif_priv_data(env);
+    Encoder* e;
+    TermStack stack;
 
     ERL_NIF_TERM ret = 0;
 
-    ERL_NIF_TERM stack;
     ERL_NIF_TERM curr;
     ERL_NIF_TERM item;
     const ERL_NIF_TERM* tuple;
+    ERL_NIF_TERM tmp_argv[3];
     int arity;
     ErlNifSInt64 lval;
     double dval;
 
-    size_t start;
-    size_t bytes_written = 0;
+    void* res;
 
-    if(argc != 3) {
-        return enif_make_badarg(env);
-    } else if(!enif_get_resource(env, argv[0], st->res_enc, (void**) &e)) {
-        return enif_make_badarg(env);
-    } else if(!enif_is_list(env, argv[1])) {
-        return enif_make_badarg(env);
-    } else if(!enif_is_list(env, argv[2])) {
+    size_t start;
+    size_t bytes_processed = 0;
+
+    if(!enif_get_resource(env, argv[0], st->res_enc, &res)) {
         return enif_make_badarg(env);
     }
 
+    e = (Encoder*) res;
+
     if(!enc_init(e, env)) {
         return enif_make_badarg(env);
     }
 
-    stack = argv[1];
+    if(!termstack_restore(env, argv[1], &stack)) {
+        return enif_make_badarg(env);
+    }
+
     e->iolist = argv[2];
 
     start = e->iosize + e->i;
 
-    while(!enif_is_empty_list(env, stack)) {
+    while(!termstack_is_empty(&stack)) {
+        bytes_processed = (e->iosize + e->i) - start;
 
-        bytes_written += (e->iosize + e->i) - start;
+        if(should_yield(bytes_processed, e->bytes_per_red)) {
 
-        if(should_yield(env, &bytes_written, e->bytes_per_red)) {
-            return enif_make_tuple4(
+            assert(enif_is_list(env, e->iolist));
+
+            tmp_argv[0] = argv[0];
+            tmp_argv[1] = termstack_save(env, &stack);
+            tmp_argv[2] = e->iolist;
+
+            termstack_destroy(&stack);
+            bump_used_reds(env, bytes_processed, e->bytes_per_red);
+
+#if SCHEDULE_NIF_PRESENT
+            return enif_schedule_nif(
+                    env,
+                    "nif_encode_iter",
+                    0,
+                    encode_iter,
+                    3,
+                    tmp_argv
+                );
+#else
+            return enif_make_tuple2(
                     env,
                     st->atom_iter,
-                    argv[0],
-                    stack,
-                    e->iolist
+                    enif_make_tuple_from_array(env, tmp_argv, 3)
                 );
+#endif
         }
 
-        if(!enif_get_list_cell(env, stack, &curr, &stack)) {
-            ret = enc_error(e, "internal_error");
-            goto done;
-        }
-        if(enif_is_identical(curr, e->atoms->ref_object)) {
-            if(!enif_get_list_cell(env, stack, &curr, &stack)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            if(enif_is_empty_list(env, curr)) {
-                if(!enc_end_object(e)) {
+        curr = termstack_pop(&stack);
+
+        if(enif_is_atom(env, curr)) {
+            if(enif_is_identical(curr, e->atoms->ref_object)) {
+                curr = termstack_pop(&stack);
+
+                if(!enif_get_list_cell(env, curr, &item, &curr)) {
+                    if(!enc_end_object(e)) {
+                        ret = enc_error(e, "internal_error");
+                        goto done;
+                    }
+                    continue;
+                }
+                if(!enif_get_tuple(env, item, &arity, &tuple)) {
+                    ret = enc_obj_error(e, "invalid_object_member", item);
+                    goto done;
+                }
+                if(arity != 2) {
+                    ret = enc_obj_error(e, "invalid_object_member_arity", item);
+                    goto done;
+                }
+                if(!enc_comma(e)) {
                     ret = enc_error(e, "internal_error");
                     goto done;
                 }
-                continue;
-            }
-            if(!enif_get_list_cell(env, curr, &item, &curr)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            if(!enif_get_tuple(env, item, &arity, &tuple)) {
-                ret = enc_obj_error(e, "invalid_object_member", item);
-                goto done;
-            }
-            if(arity != 2) {
-                ret = enc_obj_error(e, "invalid_object_member_arity", item);
-                goto done;
-            }
-            if(!enc_comma(e)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            if(!enc_string(e, tuple[0])) {
-                ret = enc_obj_error(e, "invalid_object_member_key", tuple[0]);
-                goto done;
-            }
-            if(!enc_colon(e)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            stack = enif_make_list_cell(env, curr, stack);
-            stack = enif_make_list_cell(env, e->atoms->ref_object, stack);
-            stack = enif_make_list_cell(env, tuple[1], stack);
-        } else if(enif_is_identical(curr, e->atoms->ref_array)) {
-            if(!enif_get_list_cell(env, stack, &curr, &stack)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            if(enif_is_empty_list(env, curr)) {
-                if(!enc_end_array(e)) {
+                if(!enc_object_key(env, e, tuple[0])) {
+                    ret = enc_obj_error(e, "invalid_object_member_key", tuple[0]);
+                    goto done;
+                }
+                if(!enc_colon(e)) {
                     ret = enc_error(e, "internal_error");
                     goto done;
                 }
-                continue;
-            }
-            if(!enc_comma(e)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            if(!enif_get_list_cell(env, curr, &item, &curr)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            stack = enif_make_list_cell(env, curr, stack);
-            stack = enif_make_list_cell(env, e->atoms->ref_array, stack);
-            stack = enif_make_list_cell(env, item, stack);
-        } else if(enif_compare(curr, e->atoms->atom_null) == 0) {
-            if(!enc_literal(e, "null", 4)) {
-                ret = enc_error(e, "null");
-                goto done;
-            }
-        } else if(e->use_nil && enif_compare(curr, e->atoms->atom_nil) == 0) {
-            if(!enc_literal(e, "null", 4)) {
-                ret = enc_error(e, "null");
-                goto done;
-            }
-        } else if(enif_compare(curr, e->atoms->atom_true) == 0) {
-            if(!enc_literal(e, "true", 4)) {
-                ret = enc_error(e, "true");
-                goto done;
-            }
-        } else if(enif_compare(curr, e->atoms->atom_false) == 0) {
-            if(!enc_literal(e, "false", 5)) {
-                ret = enc_error(e, "false");
-                goto done;
-            }
-        } else if(enif_is_binary(env, curr)) {
-            if(!enc_string(e, curr)) {
+
+                termstack_push(&stack, curr);
+                termstack_push(&stack, e->atoms->ref_object);
+                termstack_push(&stack, tuple[1]);
+            } else if(enif_is_identical(curr, e->atoms->ref_array)) {
+                curr = termstack_pop(&stack);
+
+                if(!enif_get_list_cell(env, curr, &item, &curr)) {
+                    if(!enc_end_array(e)) {
+                        ret = enc_error(e, "internal_error");
+                        goto done;
+                    }
+                    continue;
+                }
+                if(!enc_comma(e)) {
+                    ret = enc_error(e, "internal_error");
+                    goto done;
+                }
+
+                termstack_push(&stack, curr);
+                termstack_push(&stack, e->atoms->ref_array);
+                termstack_push(&stack, item);
+            } else if(enif_is_identical(curr, e->atoms->atom_null)) {
+                if(!enc_literal(e, "null", 4)) {
+                    ret = enc_error(e, "null");
+                    goto done;
+                }
+            } else if(e->use_nil && enif_is_identical(curr, e->atoms->atom_nil)) {
+                if(!enc_literal(e, "null", 4)) {
+                    ret = enc_error(e, "null");
+                    goto done;
+                }
+            } else if(enif_is_identical(curr, e->atoms->atom_true)) {
+                if(!enc_literal(e, "true", 4)) {
+                    ret = enc_error(e, "true");
+                    goto done;
+                }
+            } else if(enif_is_identical(curr, e->atoms->atom_false)) {
+                if(!enc_literal(e, "false", 5)) {
+                    ret = enc_error(e, "false");
+                    goto done;
+                }
+            } else if(!enc_atom(e, curr)) {
                 ret = enc_obj_error(e, "invalid_string", curr);
                 goto done;
             }
-        } else if(enif_is_atom(env, curr)) {
+        } else if(enif_is_binary(env, curr)) {
             if(!enc_string(e, curr)) {
                 ret = enc_obj_error(e, "invalid_string", curr);
                 goto done;
@@ -792,17 +868,13 @@ encode_iter(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
                 ret = enc_error(e, "internal_error");
                 goto done;
             }
-            if(enif_is_empty_list(env, tuple[0])) {
+            if(!enif_get_list_cell(env, tuple[0], &item, &curr)) {
                 if(!enc_end_object(e)) {
                     ret = enc_error(e, "internal_error");
                     goto done;
                 }
                 continue;
             }
-            if(!enif_get_list_cell(env, tuple[0], &item, &curr)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
             if(!enif_get_tuple(env, item, &arity, &tuple)) {
                 ret = enc_obj_error(e, "invalid_object_member", item);
                 goto done;
@@ -811,7 +883,7 @@ encode_iter(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
                 ret = enc_obj_error(e, "invalid_object_member_arity", item);
                 goto done;
             }
-            if(!enc_string(e, tuple[0])) {
+            if(!enc_object_key(env, e, tuple[0])) {
                 ret = enc_obj_error(e, "invalid_object_member_key", tuple[0]);
                 goto done;
             }
@@ -819,36 +891,34 @@ encode_iter(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
                 ret = enc_error(e, "internal_error");
                 goto done;
             }
-            stack = enif_make_list_cell(env, curr, stack);
-            stack = enif_make_list_cell(env, e->atoms->ref_object, stack);
-            stack = enif_make_list_cell(env, tuple[1], stack);
-#if MAP_TYPE_PRESENT
+
+            termstack_push(&stack, curr);
+            termstack_push(&stack, e->atoms->ref_object);
+            termstack_push(&stack, tuple[1]);
         } else if(enif_is_map(env, curr)) {
             if(!enc_map_to_ejson(env, curr, &curr)) {
                 ret = enc_error(e, "internal_error");
                 goto done;
             }
-            stack = enif_make_list_cell(env, curr, stack);
-#endif
+
+            termstack_push(&stack, curr);
         } else if(enif_is_list(env, curr)) {
             if(!enc_start_array(e)) {
                 ret = enc_error(e, "internal_error");
                 goto done;
             }
-            if(enif_is_empty_list(env, curr)) {
+
+            if(!enif_get_list_cell(env, curr, &item, &curr)) {
                 if(!enc_end_array(e)) {
                     ret = enc_error(e, "internal_error");
                     goto done;
                 }
                 continue;
             }
-            if(!enif_get_list_cell(env, curr, &item, &curr)) {
-                ret = enc_error(e, "internal_error");
-                goto done;
-            }
-            stack = enif_make_list_cell(env, curr, stack);
-            stack = enif_make_list_cell(env, e->atoms->ref_array, stack);
-            stack = enif_make_list_cell(env, item, stack);
+
+            termstack_push(&stack, curr);
+            termstack_push(&stack, e->atoms->ref_array);
+            termstack_push(&stack, item);
         } else {
             if(!enc_unknown(e, curr)) {
                 ret = enc_error(e, "internal_error");
@@ -857,18 +927,22 @@ encode_iter(ErlNifEnv* env, int argc, const ERL_NIF_TERM argv[])
         }
     }
 
-    if(!enc_done(e, &item)) {
+    if(!enc_flush(e)) {
         ret = enc_error(e, "internal_error");
         goto done;
     }
 
-    if(e->iolen == 0) {
-        ret = item;
+    assert(enif_is_list(env, e->iolist));
+
+    if(e->partial_output) {
+        ret = enif_make_tuple2(env, e->atoms->atom_partial, e->iolist);
     } else {
-        ret = enif_make_tuple2(env, e->atoms->atom_partial, item);
+        ret = e->iolist;
     }
 
 done:
+    bump_used_reds(env, bytes_processed, e->bytes_per_red);
+    termstack_destroy(&stack);
 
     return ret;
 }
