@@ -11,10 +11,13 @@
 # the License.
 
 import json
+import random
+import string
 import time
 import unittest
 import uuid
 import os
+import threading
 
 import requests
 
@@ -23,17 +26,30 @@ import user_docs
 import limit_docs
 
 
+COUCH_HOST = os.environ.get("COUCH_HOST") or "http://127.0.0.1:15984"
+COUCH_USER = os.environ.get("COUCH_USER")
+COUCH_PASS = os.environ.get("COUCH_PASS")
+
+
+BULK_BATCH_SIZE = 500
+
+
 def random_db_name():
     return "mango_test_" + uuid.uuid4().hex
 
 
+def random_string(n_max):
+    n = random.choice(range(n_max))
+    return "".join(random.choice(string.ascii_letters) for _ in range(n))
+
+
 def has_text_service():
-    return os.environ.get("MANGO_TEXT_INDEXES") == "1"
+    features = requests.get(COUCH_HOST).json()["features"]
+    return "search" in features
 
 
-def get_from_environment(key, default):
-    value = os.environ.get(key)
-    return value if value is not None else default
+def clean_up_dbs():
+    return not os.environ.get("MANGO_TESTS_KEEP_DBS")
 
 
 # add delay functionality
@@ -42,62 +58,73 @@ def delay(n=5, t=0.5):
         time.sleep(t)
 
 
+class Concurrently(object):
+    def __init__(self, thread, thread_args, start=True):
+        self.thread = threading.Thread(target=self.wrapper, args=(thread, thread_args))
+        self.return_value = None
+        if start:
+            self.start()
+
+    def wrapper(self, body, args):
+        self.return_value = body(*args)
+
+    def start(self):
+        self.thread.start()
+
+    def get_result(self):
+        self.thread.join()
+        return self.return_value
+
+    def join(self):
+        self.thread.join()
+
+
 class Database(object):
     def __init__(
         self,
         dbname,
-        host="127.0.0.1",
-        port="15984",
-        user="testuser",
-        password="testpass",
     ):
-        root_url = get_from_environment("COUCH_HOST", "http://{}:{}".format(host, port))
-        auth_header = get_from_environment("COUCH_AUTH_HEADER", None)
-        user = get_from_environment("COUCH_USER", user)
-        password = get_from_environment("COUCH_PASSWORD", password)
-
-        self.root_url = root_url
         self.dbname = dbname
         self.sess = requests.session()
-
-        # allow explicit auth header to be set to enable testing
-        # against deployments where basic auth isn't available
-        if auth_header is not None:
-            self.sess.headers["Authorization"] = auth_header
-        else:
-            self.sess.auth = (user, password)
-
+        self.sess.auth = (COUCH_USER, COUCH_PASS)
         self.sess.headers["Content-Type"] = "application/json"
 
     @property
     def url(self):
-        return "{}/{}".format(self.root_url, self.dbname)
+        return "{}/{}".format(COUCH_HOST, self.dbname)
 
     def path(self, parts):
         if isinstance(parts, ("".__class__, "".__class__)):
             parts = [parts]
         return "/".join([self.url] + parts)
 
-    def create(self, q=1, n=1):
+    def create(self, q=1, n=1, partitioned=False):
         r = self.sess.get(self.url)
         if r.status_code == 404:
-            r = self.sess.put(self.url, params={"q": q, "n": n})
+            p = str(partitioned).lower()
+            r = self.sess.put(self.url, params={"q": q, "n": n, "partitioned": p})
             r.raise_for_status()
 
     def delete(self):
         r = self.sess.delete(self.url)
 
     def recreate(self):
-        r = self.sess.get(self.url)
-        if r.status_code == 200:
-            db_info = r.json()
-            docs = db_info["doc_count"] + db_info["doc_del_count"]
-            if docs == 0:
-                # db never used - create unnecessary
-                return
-            self.delete()
-        self.create()
-        self.recreate()
+        NUM_TRIES = 10
+
+        for k in range(NUM_TRIES):
+            r = self.sess.get(self.url)
+            if r.status_code == 200:
+                db_info = r.json()
+                docs = db_info["doc_count"] + db_info["doc_del_count"]
+                if docs == 0:
+                    # db exists and it is empty -- exit condition is met
+                    return
+                self.delete()
+            self.create()
+            time.sleep(k * 0.1)
+        raise Exception(
+            "Failed to recreate the database after {} tries".format(NUM_TRIES)
+        )
 
     def save_doc(self, doc):
         self.save_docs([doc])
@@ -108,12 +135,14 @@ class Database(object):
         r.raise_for_status()
 
     def save_docs(self, docs, **kwargs):
-        body = json.dumps({"docs": docs})
-        r = self.sess.post(self.path("_bulk_docs"), data=body, params=kwargs)
-        r.raise_for_status()
-        for doc, result in zip(docs, r.json()):
-            doc["_id"] = result["id"]
-            doc["_rev"] = result["rev"]
+        for offset in range(0, len(docs), BULK_BATCH_SIZE):
+            chunk = docs[offset : (offset + BULK_BATCH_SIZE)]
+            body = {"docs": chunk}
+            r = self.sess.post(self.path("_bulk_docs"), json=body, params=kwargs)
+            r.raise_for_status()
+            for doc, result in zip(chunk, r.json()):
+                doc["_id"] = result["id"]
+                doc["_rev"] = result["rev"]
 
     def open_doc(self, docid):
         r = self.sess.get(self.path(docid))
@@ -252,6 +281,7 @@ class Database(object):
         return_raw=False,
         update=True,
         executionStats=False,
+        partition=None,
     ):
         body = {
             "selector": selector,
@@ -272,10 +302,14 @@ class Database(object):
         if executionStats == True:
             body["execution_stats"] = True
         body = json.dumps(body)
-        if explain:
-            path = self.path("_explain")
+        if partition:
+            ppath = "_partition/{}/".format(partition)
         else:
-            path = self.path("_find")
+            ppath = ""
+        if explain:
+            path = self.path("{}_explain".format(ppath))
+        else:
+            path = self.path("{}_find".format(ppath))
         r = self.sess.post(path, data=body)
         r.raise_for_status()
         if explain or return_raw:
@@ -299,15 +333,25 @@ class UsersDbTests(unittest.TestCase):
         klass.db = Database("_users")
         user_docs.setup_users(klass.db)
 
+    @classmethod
+    def tearDownClass(klass):
+        if clean_up_dbs():
+            klass.db.delete()
+
     def setUp(self):
         self.db = self.__class__.db
 
 
 class DbPerClass(unittest.TestCase):
     @classmethod
-    def setUpClass(klass):
+    def setUpClass(klass, partitioned=False):
         klass.db = Database(random_db_name())
-        klass.db.create(q=1, n=1)
+        klass.db.create(q=1, n=1, partitioned=partitioned)
+
+    @classmethod
+    def tearDownClass(klass):
+        if clean_up_dbs():
+            klass.db.delete()
 
     def setUp(self):
         self.db = self.__class__.db
@@ -322,6 +366,15 @@ class UserDocsTests(DbPerClass):
         user_docs.setup(klass.db)
 
 
+class PartitionedUserDocsTests(DbPerClass):
+    INDEX_TYPE = "json"
+
+    @classmethod
+    def setUpClass(klass):
+        super(PartitionedUserDocsTests, klass).setUpClass(partitioned=True)
+        user_docs.setup(klass.db, partitioned=True)
+
+
 class UserDocsTestsNoIndexes(DbPerClass):
     INDEX_TYPE = "special"
 
@@ -331,6 +384,15 @@ class UserDocsTestsNoIndexes(DbPerClass):
         user_docs.setup(klass.db, index_type=klass.INDEX_TYPE)
 
 
+class PartitionedUserDocsTestsNoIndexes(DbPerClass):
+    INDEX_TYPE = "special"
+
+    @classmethod
+    def setUpClass(klass):
+        super(PartitionedUserDocsTestsNoIndexes, klass).setUpClass(partitioned=True)
+        user_docs.setup(klass.db, index_type=klass.INDEX_TYPE, partitioned=True)
+
+
 class UserDocsTextTests(DbPerClass):
     INDEX_TYPE = "text"
     DEFAULT_FIELD = None
