@@ -84,6 +84,7 @@ defmodule NouveauTest do
             function (doc) {
               index("string", "foo", doc.foo, {store: true});
               index("double", "bar", doc.bar, {store: true});
+              index("stored", "baz", doc.foo);
             }
           """
         }
@@ -169,6 +170,7 @@ defmodule NouveauTest do
     assert Map.get(info, "disk_size") > 0
     assert Map.get(info, "num_docs") > 0
     assert Map.get(info, "update_seq") > 0
+    assert String.length(Map.get(info, "signature")) > 0
   end
 
   @tag :with_db
@@ -251,6 +253,37 @@ defmodule NouveauTest do
     assert ids == ["doc2"]
   end
 
+  @tag :with_db
+  test "multiple values for stored field", context do
+    db_name = context[:db_name]
+    resp = Couch.post("/#{db_name}/_bulk_docs",
+      headers: ["Content-Type": "application/json"],
+      body: %{:docs => [
+        %{"_id" => "doc1", "colors" =>
+           ["red", "orange", "yellow", "green", "blue", "indigo", "violet"]}]}
+    )
+    assert resp.status_code in [201]
+    create_ddoc(db_name,
+      %{
+        nouveau: %{
+          bar: %{
+            default_analyzer: "standard",
+            index: """
+            function (doc) {
+            doc.colors.forEach((color) =>
+              index('string', 'color', color, {'store': true}));
+            }
+            """
+          }
+        }
+      })
+    url = "/#{db_name}/_design/foo/_nouveau/bar"
+    resp = Couch.post(url, body: %{q: "*:*", include_docs: true})
+    assert_status_code(resp, 200)
+    assert Enum.at(resp.body["hits"], 0)["doc"]["colors"] ==
+      ["red", "orange", "yellow", "green", "blue", "indigo", "violet"]
+  end
+
   @tag :with_db
   test "sort by string field (asc)", context do
     db_name = context[:db_name]
@@ -348,6 +381,23 @@ defmodule NouveauTest do
     assert ranges == %{"bar" => %{"cheap" => 3, "expensive" => 1}}
   end
 
+  @tag :with_db
+  test "top_n", context do
+    db_name = context[:db_name]
+    create_search_docs(db_name)
+    create_ddoc(db_name)
+
+    url = "/#{db_name}/_design/foo/_nouveau/bar"
+    resp = Couch.post(url, body: %{q: "*:*", ranges: %{bar: [
+      %{label: "cheap", max: 42},
+      %{label: "expensive", min: 42, min_inclusive: false}]},
+      top_n: 1,
+      include_docs: true})
+    assert_status_code(resp, 200)
+    %{:body => %{"ranges" => ranges}} = resp
+    assert ranges == %{"bar" => %{"cheap" => 3}}
+  end
+
   @tag :with_db
   test "mango search by number", context do
     db_name = context[:db_name]
@@ -549,15 +599,15 @@ defmodule NouveauTest do
 
     assert hit1["doc"]["_id"] == "doc1"
     assert hit1["doc"]["_rev"] == "1-a"
-    assert hit1["fields"] == %{"bar" => 0.0, "foo" => "baz"}
+    assert hit1["fields"] == %{"bar" => 0.0, "foo" => "baz", "baz" => "baz"}
 
     assert hit2["doc"]["_id"] == "doc3"
     assert hit2["doc"]["_rev"] == "2-c"
-    assert hit2["fields"] == %{"bar" => 13.0, "foo" => "barX"}
+    assert hit2["fields"] == %{"bar" => 13.0, "foo" => "barX", "baz" => "barX"}
 
     assert hit3["doc"]["_id"] == "doc4"
     assert hit3["doc"]["_rev"] == "1-b"
-    assert hit3["fields"] == %{"bar" => 43.0, "foo" => "fooX"}
+    assert hit3["fields"] == %{"bar" => 43.0, "foo" => "fooX", "baz" => "fooX"}
 
     # purge docs
     purge_body = %{
@@ -579,7 +629,7 @@ defmodule NouveauTest do
     # doc1: 2-c deleted was purged, 1-a is still the winner
     assert hit1["doc"]["_id"] == "doc1"
     assert hit1["doc"]["_rev"] == "1-a"
-    assert hit1["fields"] ==  %{"bar" => 0.0, "foo" => "baz"}
+    assert hit1["fields"] ==  %{"bar" => 0.0, "foo" => "baz", "baz" => "baz"}
 
     # doc2: doc was deleted and now it's completely purged
 
@@ -588,7 +638,7 @@ defmodule NouveauTest do
     # doc4: 2-c was purged, 1-a is the new winner
     assert hit2["doc"]["_id"] == "doc4"
     assert hit2["doc"]["_rev"] == "1-a"
-    assert hit2["fields"] == %{"bar" => 42.0, "foo" => "foo"}
+    assert hit2["fields"] == %{"bar" => 42.0, "foo" => "foo", "baz" => "foo"}
 
     resp = Couch.get("/#{db_name}")
     db_purge_seq = resp.body["purge_seq"]
@@ -622,6 +672,28 @@ defmodule NouveauTest do
     assert_status_code(resp, 400)
   end
 
+  @tag :with_db
+  test "index not found", context do
+    db_name = context[:db_name]
+    create_search_docs(db_name)
+    create_ddoc(db_name)
+    url = "/#{db_name}/_design/foo/_nouveau/missing"
+    resp = Couch.post(url, body: %{q: "foo:bar", include_docs: true})
+    assert_status_code(resp, 404)
+  end
+
+  @tag :with_db
+  test "meta", context do
+    db_name = context[:db_name]
+    create_search_docs(db_name)
+    create_ddoc(db_name)
+
+    url = "/#{db_name}/_design/foo/_nouveau/bar"
+    resp = Couch.get(url, query: %{q: "*:*", include_docs: true})
+    assert_status_code(resp, 200)
+    assert resp.body["update_latency"] > 0
+  end
+
   def seq(str) do
     String.to_integer(hd(Regex.run(~r/^[0-9]+/, str)))
   end
